@@ -8,14 +8,19 @@ import h5py
 import pickle
 import logging
 import argparse
+import warnings
 import numpy as np
 import pandas as pd
+import Bio.Seq
 
 if __name__ != '__main__':
     from ._version import __version__
-    import inStrain.profileUtilities
+    #import inStrain.profileUtilities
 
-import inStrain.profileUtilities
+#import inStrain.profileUtilities
+import inStrain.profile.profile_utilities
+import inStrain.logUtils
+import inStrain.filter_reads
 
 class SNVprofile:
     '''
@@ -156,6 +161,199 @@ class SNVprofile:
         else:
             logging.error('I dont know the location of {0}!'.format(name))
 
+    def generate(self, name, store=True, return_table=False, **kwargs):
+        '''
+        Generate user-facing output tables based on information stored in the IS
+        '''
+
+        report_mm_level = kwargs.get('mm_level', False)
+        if name == 'SNVs':
+            column_order = ['scaffold', 'position', 'position_coverage', 'allele_count',
+                            'ref_base', 'con_base', 'var_base',
+                            'ref_freq', 'con_freq', 'var_freq',
+                            'A', 'C', 'T', 'G',
+                            'gene', 'mutation', 'mutation_type', 'cryptic']
+
+            # Get the base table
+            db = self.get_nonredundant_snv_table()
+
+            # Do you have the mutation types? If so, merge in
+            mdb = self.get('SNP_mutation_types')
+            if (mdb is not None):
+                if len(mdb) > 0:
+                    mdb = mdb[['scaffold', 'position', 'mutation_type', 'mutation', 'gene']]
+                    db = pd.merge(db, mdb, how='left', on=['scaffold', 'position'])
+
+            # Do you have information about coverage? If so, merge in
+            pass
+
+            db = reorder_columns(db, column_order)
+
+        elif name == 'scaffold_info':
+            column_order = ['scaffold', 'length', 'coverage', 'breadth',
+                            'nucl_diversity',
+                            'coverage_median', 'coverage_std', 'coverage_SEM',
+                            'breadth_minCov', 'breadth_expected',
+                            'nucl_diversity_median',
+                            'nucl_diversity_rarefied', 'nucl_diversity_rarefied_median',
+                            'breadth_rarefied',
+                            'conANI_reference', 'popANI_reference',
+                            'SNS_count', 'SNV_count', 'divergent_site_count']
+
+            db = self.get_nonredundant_scaffold_table()
+            db = reorder_columns(db, column_order)
+
+        elif name == 'linkage':
+            column_order = ['scaffold', 'position_A', 'position_B', 'distance',
+                            'r2', 'd_prime',
+                            'r2_normalized', 'd_prime_normalized',
+                            'allele_A', 'allele_a',
+                            'allele_B', 'allele_b',
+                            'countab', 'countAb', 'countaB', 'countAB', 'total']
+            db = self.get_nonredundant_linkage_table()
+            db = reorder_columns(db, column_order)
+
+        elif name == 'gene_info':
+            column_order = ['scaffold', 'gene', 'gene_length',
+                            'coverage', 'breadth', 'breadth_minCov', 'nucl_diversity',
+                             'start', 'end', 'direction', 'partial',
+                             'dNdS_substitutions', 'pNpS_variants',
+                             'SNV_count', 'SNV_S_count', 'SNV_N_count',
+                             'SNS_count', 'SNS_S_count', 'SNS_N_count',
+                             'divergent_site_count']
+
+            Gdb = self.get('genes_table')
+            if Gdb is None:
+                logging.info("Cannot generate genes_table, no genes were profiled")
+                return
+
+            for thing in ['genes_coverage', 'genes_clonality', 'genes_SNP_count']:
+                db = self.get(thing)
+                if db is None:
+                    logging.debug('Skipping {0} gene calculation; you have none'.format(thing))
+                    continue
+                if len(db) == 0:
+                    logging.debug('Skipping {0} gene calculation; you have none'.format(thing))
+                    continue
+                db = db.sort_values('mm').drop_duplicates(subset=['gene'], keep='last')
+                del db['mm']
+                Gdb = pd.merge(Gdb, db, on='gene', how='left')
+
+            db = Gdb
+            for c in ['N_sites', 'S_sites']:
+                if c in db.columns:
+                    del db[c]
+
+            db = reorder_columns(db, column_order)
+            db = db[db['coverage'] > 0]
+
+        elif name == 'genome_info':
+            column_order = ['genome', 'coverage', 'breadth', 'nucl_diversity',
+                            'length', 'true_scaffolds', 'detected_scaffolds',
+                            'coverage_median', 'coverage_std', 'coverage_SEM',
+                            'breadth_minCov', 'breadth_expected',
+                            'nucl_diversity_rarefied',
+                            'conANI_reference', 'popANI_reference',
+                            'iRep', 'iRep_GC_corrected',
+                            'linked_SNV_count', 'SNV_distance_mean', 'r2_mean','d_prime_mean',
+                            'consensus_divergent_sites',
+                            'population_divergent_sites',
+                            'SNS_count', 'SNV_count',
+                            'filtered_read_pair_count',
+                            'reads_unfiltered_pairs',
+                            'reads_mean_PID']
+
+            db = self.get('genome_level_info')
+            db = reorder_columns(db, column_order)
+
+            # Get rid of some of those dumb columns
+            read_columns = [r for r in db.columns if r.startswith('reads_')]
+            keep_columns = [r for r in read_columns if r in ['reads_unfiltered_reads',
+                            'reads_unfiltered_pairs', 'reads_mean_PID']]
+            for col in set(read_columns) - set(keep_columns):
+                if col in db:
+                    del db[col]
+
+            if not report_mm_level:
+                if 'mm' in db.columns:
+                    db = db.sort_values('mm').drop_duplicates(
+                            subset=['genome'], keep='last')\
+                            .sort_values('genome')
+                    del db['mm']
+
+        elif name == 'mapping_info':
+            column_order = ['scaffold', 'pass_pairing_filter', 'filtered_pairs']
+
+            db = self.get('mapping_info')
+            values = inStrain.filter_reads.write_mapping_info(db, None, **kwargs)
+
+            if store:
+                base = self.get_output_base()
+                location = base + name + '.tsv'
+                os.remove(location) if os.path.exists(location) else None
+                f = open(location, 'a')
+                f.write("# {0}\n".format(' '.join(["{0}:{1}".format(k, v) for k, v in values.items()])))
+
+                db = reorder_columns(db, column_order)
+                db.to_csv(f, index=False, sep='\t')
+
+                f.close()
+
+            if return_table:
+                return db
+            else:
+                return
+
+        elif name == 'comparisonsTable':
+            db = self.get_nonredundant_RC_table()
+
+        elif name == 'pairwise_SNP_locations':
+            column_order = ['mm', 'scaffold', 'position',
+            'name1', 'name2',
+            'consensus_SNP', 'population_SNP',
+            'con_base_1', 'ref_base_1', 'var_base_1', 'position_coverage_1',
+            'A_1', 'C_1', 'T_1', 'G_1',
+            'con_base_2', 'ref_base_2', 'var_base_2', 'position_coverage_2',
+            'A_2', 'C_2', 'T_2', 'G_2']
+
+            db = self.get('pairwise_SNP_locations')
+            db = reorder_columns(db, column_order)
+
+            if ((not report_mm_level) & (len(db) > 0)):
+                db = db.sort_values('mm')\
+                        .drop_duplicates(subset=['scaffold', 'position',
+                                        'name1', 'name2'], keep='last')\
+                        .sort_index().drop(columns=['mm'])
+
+        else:
+            logging.error("Do not know how to store {0}! Crashing now".format(name))
+            assert False
+
+        if store:
+            out_base = self.get_output_base()
+            db.to_csv(out_base + name + '.tsv', index=False, sep='\t')
+
+        if return_table:
+            return db
+
+    def get_parsed_log(self, most_recent=True):
+        logloc = os.path.join(self.get_location('log'), 'log.log')
+        Ldb = inStrain.logUtils.load_log(logloc)
+
+        # Filter the log
+        if most_recent:
+            Ldb = inStrain.logUtils.filter_most_recent(Ldb)
+
+        return Ldb
+
+    def get_output_base(self):
+        return self.get_location('output') + \
+                    os.path.basename(self.get('location')) + '_'
+
+    def get_read_length(self):
+        Rdb = self.get('mapping_info').head(1)
+        return float(Rdb.loc[0, 'mean_pair_length'])
+
     def verify(self):
         '''
         Run a series of tests to make sure everything is kosher
@@ -225,7 +423,7 @@ class SNVprofile:
             dbs = []
             scaff2clonT = clonT
             for scaff, clonT in scaff2clonT.items():
-                db = inStrain.profileUtilities._clonT_to_table(clonT)
+                db = inStrain.profile.profile_utilities._clonT_to_table(clonT)
                 db['scaffold'] = scaff
                 dbs.append(db)
 
@@ -288,8 +486,14 @@ class SNVprofile:
             Adb = self._get_attributes_file()
 
             if not same_versions(Adb.loc['version', 'value'], __version__):
-                logging.error("Warning! Your inStrain folder is from version {0}, while the installed version is {1}.\nIf you experience weird behavior, this might be why".format(
-                        Adb.loc['version', 'value'], __version__))
+                error_message = "Warning! Your inStrain folder is from version {0}, while the installed version is {1}.\nIf you experience weird behavior, this might be why".format(
+                        Adb.loc['version', 'value'], __version__)
+
+                # This is needed because otherwise will make a logger in error!
+                if logging.getLogger('').handlers:
+                    logging.error(error_message)
+                else:
+                    print(error_message)
 
             if self.location != self.get('location'):
                 self.store('location', self.location, 'value', 'Location of SNVprofile object')
@@ -302,13 +506,13 @@ class SNVprofile:
         Aloc = str(os.path.join(self.location, 'raw_data/attributes.tsv')).strip()
         # This crazy mumbo-jumbo is to prevent a failure when trying to read while its being written
         for i in range(0,100):
-            while True:
-                try:
-                    Adb = pd.read_csv(Aloc, sep='\t', index_col='name')
-                except:
-                    logging.error("Cannot load {0}; try {1}".format(Aloc, i))
-                    time.sleep(1)
+            # while True:
+            try:
+                Adb = pd.read_csv(Aloc, sep='\t', index_col='name')
                 break
+            except:
+                logging.error("Cannot load {0}; try {1}".format(Aloc, i))
+                time.sleep(1)
 
         return Adb
 
@@ -502,10 +706,19 @@ class SNVprofile:
         return loaded
 
     def _store_pandas(self, obj, location):
-        obj.to_csv(location)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            obj.to_csv(location)
 
     def _load_pandas(self, location):
-        return pd.read_csv(location, index_col=0)
+        # This stupid thing is required because just calling the read_csv sometimes causes the warning
+        # /home/mattolm/.pyenv/versions/3.8.2/envs/3.8.2_testInstrain/lib/python3.8/site-packages/numpy/lib/
+        # arraysetops.py:569: FutureWarning: elementwise comparison failed; returning scalar
+        # instead, but in the future will perform elementwise comparison
+        # mask |= (ar1 == a)"
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            return pd.read_csv(location, index_col=0)
 
     def _store_pickle(self, obj, location):
         f = open(location, 'wb')
@@ -529,7 +742,7 @@ class SNVprofile:
 
     def _load_dictionary(self, location):
         '''
-        Store a dictionary using json
+        Load a dictionary using json
         '''
         with open(location, 'r') as fp:
             return(json.load(fp))
@@ -596,6 +809,7 @@ def load_scaff2pair2mm2SNPs(location, scaffolds=[], pairs=[]):
 
 def _json_helper(o):
     if isinstance(o, np.int64): return int(o)
+    if isinstance(o, Bio.Seq.Seq): return str(o)
     raise TypeError
 
 class SNVprofile_old:
@@ -686,7 +900,7 @@ class SNVprofile_old:
             dbs = []
             scaff2clonT = self.clonT
             for scaff, clonT in scaff2clonT.items():
-                db = inStrain.profileUtilities._clonT_to_table(clonT)
+                db = inStrain.profile.profile_utilities._clonT_to_table(clonT)
                 db['scaffold'] = scaff
                 dbs.append(db)
 
@@ -796,39 +1010,43 @@ def convert_SNVprofile(pickle_loc):
         #
 
         elif attr == 'covT':
-            new = {s:inStrain.profileUtilities.shrink_basewise(cov, 'coverage') for s, cov in getattr(oIS, attr).items()}
+            new = {s:inStrain.profile.profile_utilities.shrink_basewise(cov, 'coverage') for s, cov in getattr(oIS, attr).items()}
             nIS.store('covT', new,
                     'special', "Scaffold -> mm -> position based coverage")
-            # nIS.store('covT', inStrain.profileUtilities.shrink_basewise(getattr(oIS, attr), 'coverage'),
-            #         'special', "Scaffold -> mm -> position based coverage")
 
         elif attr == 'snpsCounted':
-            new = {s:inStrain.profileUtilities.shrink_basewise(cov, 'snpCounted') for s, cov in getattr(oIS, attr).items()}
+            new = {s:inStrain.profile.profile_utilities.shrink_basewise(cov, 'snpCounted') for s, cov in getattr(oIS, attr).items()}
             nIS.store('snpsCounted', new,
                     'special', "Scaffold -> mm -> position based True/False on if a SNPs is there")
-            # nIS.store('snpsCounted', inStrain.profileUtilities.shrink_basewise(getattr(oIS, attr), 'snpsCounted'),
-            #         'special', "Scaffold -> mm -> position based True/False on if a SNPs is there")
 
         elif attr == 'clonT':
-            new = {s:inStrain.profileUtilities.shrink_basewise(cov, 'clonality') for s, cov in getattr(oIS, attr).items()}
+            new = {s:inStrain.profile.profile_utilities.shrink_basewise(cov, 'clonality') for s, cov in getattr(oIS, attr).items()}
             nIS.store('clonT', new,
                     'special', "Scaffold -> mm -> position based clonality")
 
         elif attr == 'clonTR':
-            new = {s:inStrain.profileUtilities.shrink_basewise(cov, 'clonality') for s, cov in getattr(oIS, attr).items()}
+            new = {s:inStrain.profile.profile_utilities.shrink_basewise(cov, 'clonality') for s, cov in getattr(oIS, attr).items()}
             nIS.store('clonTR', new,
                     'special', "Scaffold -> mm -> position based clonality")
-            # nIS.store('clonT', inStrain.profileUtilities.shrink_basewise(getattr(oIS, attr), 'clonality'),
-            #         'special', "Scaffold -> mm -> position based clonality")
 
-        elif attr == 'read_report':
-            nIS.store('read_report', getattr(oIS, attr), 'pandas', "Report on reads")
+        elif attr in ['mapping_info', 'read_report']:
+            nIS.store(attr, getattr(oIS, attr), 'pandas', "Report on reads")
 
         else:
             logging.error('I dont know how to store {0}!'.format(attr))
             print(type(getattr(oIS, attr)))
             break
 
+def reorder_columns(db, column_order):
+    '''
+    Reorder columns in db baesd on column order
+
+    Any column not in the column_order will be added to the end
+    '''
+    columns = set(db.columns)
+    return db[[c for c in column_order if c in columns] \
+                + list(columns - set(column_order))]
+
 # if __name__ == '__main__':
 #     parser = argparse.ArgumentParser(description= """
 #         A quick way to convert old SNP profiles (v0.3.x) to new SNP profiles (v0.4.x)""")
