@@ -44,8 +44,8 @@ def printHelp():
     compare         -> Compare multiple inStrain profiles (popANI, coverage_overlap, etc.)
 
   Single operations:
-    profile_genes   -> Calculate gene-level metrics on an inStrain profile
-    genome_wide     -> Calculate genome-level metrics on an inStrain profile
+    profile_genes   -> Calculate gene-level metrics on an inStrain profile [DEPRECATED; PROVIDE GENES TO profile]
+    genome_wide     -> Calculate genome-level metrics on an inStrain profile [DEPRECATED; PROVIDE .stb FILES TO profile / compare]
     quick_profile   -> Quickly calculate coverage and breadth of a mapping using coverM
     filter_reads    -> Commands related to filtering reads from .bam files
     plot            -> Make figures from the results of "profile" or "compare"
@@ -64,12 +64,16 @@ def parse_args(args):
     Bflags.add_argument('-d','--debug',help='Make extra debugging output',default=False,
                         action= "store_true")
     Bflags.add_argument("-h", "--help", action="help", help="show this help message and exit")
+    Bflags.add_argument(
+        "--version",
+        action="version",
+        version="inStrain version {version}".format(version=__version__))
 
 
     # Make a parent parser for read filtering
     readfilter_parent = argparse.ArgumentParser(add_help=False)
     fiflags = readfilter_parent.add_argument_group('READ FILTERING OPTIONS')
-    fiflags.add_argument("-l", "--filter_cutoff", action="store", default=0.95, type=float, \
+    fiflags.add_argument("-l", "--min_read_ani", action="store", default=0.95, type=float, \
         help='Minimum percent identity of read pairs to consensus to use the reads. Must be >, not >=')
     fiflags.add_argument("--min_mapq", action="store", default=-1, type=int,\
         help='Minimum mapq score of EITHER read in a pair to use that pair. Must be >, not >=')
@@ -80,7 +84,7 @@ def parse_args(args):
     fiflags.add_argument("--pairing_filter", help="R|How should paired reads be handled?\n" \
         + "paired_only = Only paired reads are retained\n" \
         + 'non_discordant = Keep all paired reads and singleton reads that map to a single scaffold\n' \
-        + "all_reads = Keep all reads regardless of pairing status (NOT RECOMMENDED; weird behavior will result if two parts of a pair map to different scaffolds. See documentation for deatils)\n", \
+        + "all_reads = Keep all reads regardless of pairing status (NOT RECOMMENDED; See documentation for deatils)\n", \
         default = "paired_only", choices={'paired_only', 'non_discordant', 'all_reads'})
     fiflags.add_argument("--priority_reads", help='The location of a list ' \
         + "of reads that should be retained regardless of pairing status " \
@@ -93,7 +97,7 @@ def parse_args(args):
     fiflags = readoutput_parent.add_argument_group('READ OUTPUT OPTIONS')
     # fiflags.add_argument("-s", "--generate_sam", action="store", default=None, \
     #     help='Specify the location to write a .sam file with filtered reads only.')
-    fiflags.add_argument("--deatiled_read_report", action="store_true", default=False, help='Make a detailed read report indicating deatils about each individual mapped read')
+    fiflags.add_argument("--detailed_mapping_info", action="store_true", default=False, help='Make a detailed read report indicating deatils about each individual mapped read')
 
     # Make a parent parser for SNV calling
     variant_parent = argparse.ArgumentParser(add_help=False)
@@ -116,8 +120,14 @@ def parse_args(args):
     Rflags = geneomewide_parent.add_argument_group('GENOME WIDE OPTIONS')
     Rflags.add_argument('-s', '--stb', help="Scaffold to bin. This can be a file with each line listing a scaffold and a bin name, tab-seperated. This can also be a space-seperated list of .fasta files, with one genome per .fasta file. If nothing is provided, all scaffolds will be treated as belonging to the same genome",
                         nargs='*', default=[])
-    Rflags.add_argument('--mm_level', help="Create files on the mm level (see documentation for info)",
+
+    # Make a parent for handling mm
+    mm_parent = argparse.ArgumentParser(add_help=False)
+    Rflags = mm_parent.add_argument_group('READ ANI OPTIONS')
+    Rflags.add_argument('--mm_level', help="Create output files on the mm level (see documentation for info)",
                         action='store_true', default=False)
+    Rflags.add_argument('--skip_mm_profiling', action='store_true', default=False,\
+        help="Dont perform analysis on an mm level; saves RAM and time; impacts plots and raw_data")
 
     '''
     ####### Arguments for profile operation ######
@@ -136,22 +146,27 @@ def parse_args(args):
         help='Instead of using the fasta ID (space in header before space), use the full header. Needed for some mapping tools (including bbMap)')
 
     profile_parser = subparsers.add_parser("profile",formatter_class=SmartFormatter,\
-                    parents = [profile_parent, parent_parser, readfilter_parent, variant_parent, genes_parent, geneomewide_parent], add_help=False)
+                    parents = [profile_parent, parent_parser, readfilter_parent, readoutput_parent, variant_parent, genes_parent, geneomewide_parent, mm_parent], add_help=False)
 
     # Other Parameters
     Oflags = profile_parser.add_argument_group('PROFILE OPTIONS')
+    Oflags.add_argument('--database_mode', action='store_true', default=False,\
+        help="Set a number of parameters to values appropriate for mapping to a " \
+        + "large fasta file. Will set: --min_read_ani 0.92 --skip_mm_profiling --min_genome_coverage 1")
+    Oflags.add_argument("--min_scaffold_reads", action="store", default=1, type=int,\
+        help='Minimum number of reads mapping to a scaffold to proceed with profiling it')
+    Oflags.add_argument("--min_genome_coverage", action="store", default=0, type=float,\
+        help='Minimum number of reads mapping to a genome to proceed with profiling it. MUST profile .stb if this is set')
     Oflags.add_argument("--min_snp", action="store", default=20, \
         help='Absolute minimum number of reads connecting two SNPs to calculate LD between them.')
-    Oflags.add_argument("--min_fasta_reads", action="store", default=0, type=int,\
-        help='Minimum number of reads mapping to a scaffold to proceed with profiling it')
     Oflags.add_argument('--store_everything', action='store_true', default=False,\
         help="Store intermediate dictionaries in the pickle file; will result in significantly more RAM and disk usage")
-    Oflags.add_argument('--skip_mm_profiling', action='store_true', default=False,\
-        help="Dont perform analysis on an mm level; saves RAM and time")
     Oflags.add_argument("--scaffolds_to_profile", action="store",\
         help='Path to a file containing a list of scaffolds to profile- if provided will ONLY profile those scaffolds')
     Oflags.add_argument("--rarefied_coverage", action='store', default=50,\
         help='When calculating nucleotide diversity, also calculate a rarefied version with this much coverage')
+    Oflags.add_argument('--window_length', action='store', default=10000, type=int,\
+        help='Break scaffolds into windows of this length when profiling')
 
     # Other Parameters
     Iflags = profile_parser.add_argument_group('OTHER  OPTIONS')
@@ -173,30 +188,52 @@ def parse_args(args):
     Rflags.add_argument("-o", "--output", action="store", default='instrainComparer', \
                         help='Output prefix')
 
-
     compare_parser = subparsers.add_parser("compare",formatter_class=SmartFormatter,\
-                    parents = [compare_parent, parent_parser, variant_parent], add_help=False)
+                    parents = [compare_parent, parent_parser, geneomewide_parent, variant_parent], add_help=False)
+
+    # Database mode parameters
+    Dflags = compare_parser.add_argument_group('DATABASE MODE PARAMETERS')
+    Dflags.add_argument('--database_mode', action='store_true', help=
+                        "Using the parameters below, automatically determine which genomes are present in each Profile "
+                        "and only compare scaffolds from those genomes. All profiles must have run Profile with the same .stb")
+    Dflags.add_argument('--breadth', default=0.5, type=float, help='Minimum breadth_minCov required to count a genome present')
 
     # Other Parameters
     Oflags = compare_parser.add_argument_group('OTHER OPTIONS')
-    Oflags.add_argument("-s", "--scaffolds", action="store", \
+    Oflags.add_argument("-sc", "--scaffolds", action="store",
                         help='Location to a list of scaffolds to compare. You can also make this a .fasta file and it will load the scaffold names')
+    Oflags.add_argument("--genome", action="store",
+                        help='Run scaffolds belonging to this single genome only. Must provide an .stb file')
     Oflags.add_argument('--store_coverage_overlap', action='store_true', default=False,\
         help="Also store coverage overlap on an mm level")
     Oflags.add_argument('--store_mismatch_locations', action='store_true', default=False,\
         help="Store the locations of SNPs")
     Oflags.add_argument('--include_self_comparisons', action='store_true', default=False,\
         help="Also compare IS profiles against themself")
+    Oflags.add_argument('--skip_plot_generation', action='store_true', default=False, \
+                        help="Dont create plots at the end of the run.")
+    Oflags.add_argument('--group_length', default=10000000,
+                        help="How many bp to compare simultaneously (higher will use more RAM and run more quickly)", type=int)
 
-    Gflags = compare_parser.add_argument_group('GREEDY CLUSTERING OPTIONS [THIS SECTION IS EXPERIMENTAL!]')
-    Gflags.add_argument('--greedy_clustering', action='store_true', default=False,\
-        help="Dont do pair-wise comparisons, do greedy clustering to only find the number of clsuters. If this is set, use the parameters below as well")
-    Gflags.add_argument('--g_ani', action='store', default=0.99, type=float,\
-        help="ANI threshold for greedy clustering- put the fraction not the percentage (e.g. 0.99, not 99)")
-    Gflags.add_argument('--g_cov', action='store', default=0.99, type=float,\
-        help="Alignment coverage for greedy clustering- put the fraction not the percentage (e.g. 0.5, not 10)")
-    Gflags.add_argument('--g_mm', action='store', default=100, type=int,\
-        help="Maximum read mismatch level")
+    Cflags = compare_parser.add_argument_group('GENOME CLUSTERING OPTIONS')
+    Cflags.add_argument('-ani', "--ani_threshold", help='popANI threshold to cluster genomes at. Must provide .stb file to do so',
+                        default=0.99999, type=float)
+    Cflags.add_argument('-cov', "--coverage_treshold", help='Minimum percent_genome_compared for a genome comparison' \
+                        ' to count; if below the popANI will be set to 0.', default=0.1, type=float)
+    Cflags.add_argument("--clusterAlg", help="Algorithm used to cluster genomes (passed\
+                            to scipy.cluster.hierarchy.linkage)", default='average',
+                           choices={'single', 'complete', 'average', 'weighted', 'centroid', 'median', 'ward'})
+
+
+    # Gflags = compare_parser.add_argument_group('GREEDY CLUSTERING OPTIONS [THIS SECTION IS EXPERIMENTAL!]')
+    # Gflags.add_argument('--greedy_clustering', action='store_true', default=False,\
+    #     help="Dont do pair-wise comparisons, do greedy clustering to only find the number of clsuters. If this is set, use the parameters below as well")
+    # Gflags.add_argument('--g_ani', action='store', default=0.99, type=float,\
+    #     help="ANI threshold for greedy clustering- put the fraction not the percentage (e.g. 0.99, not 99)")
+    # Gflags.add_argument('--g_cov', action='store', default=0.99, type=float,\
+    #     help="Alignment coverage for greedy clustering- put the fraction not the percentage (e.g. 0.5, not 10)")
+    # Gflags.add_argument('--g_mm', action='store', default=100, type=int,\
+    #     help="Maximum read mismatch level")
 
     '''
     ####### Arguments for profile_genes operation ######
@@ -207,6 +244,8 @@ def parse_args(args):
     # Required positional arguments
     Rflags = genes_io.add_argument_group('INPUT / OUTPUT')
     Rflags.add_argument("-i", '--IS', help="an inStrain profile object", required=True)
+    Rflags.add_argument('--store_everything', action='store_true', default=False,\
+        help="Store gene sequences in the IS object")
 
     genes_parser = subparsers.add_parser("profile_genes",formatter_class=SmartFormatter,\
                     parents = [genes_parent, genes_io, parent_parser], add_help=False)
@@ -216,7 +255,7 @@ def parse_args(args):
     '''
     # Make a parent for profile to go above the system arguments
     genome_parser = subparsers.add_parser("genome_wide",formatter_class=SmartFormatter,\
-                    parents = [geneomewide_parent, genes_io, parent_parser], add_help=False)
+                    parents = [geneomewide_parent, genes_io, mm_parent, parent_parser], add_help=False)
 
     '''
     ####### Arguments for plot operation ######
@@ -257,24 +296,23 @@ def parse_args(args):
     quick_parent = argparse.ArgumentParser(add_help=False)
     # Required positional arguments
     Rflags = quick_parent.add_argument_group('REQUIRED')
-    Rflags.add_argument('-b', '--bam', help="A bam file to profile",
-                        required=True)
-    Rflags.add_argument('-f', '--fasta', help="The .fasta file to profile",
-                        required=True)
-    Rflags.add_argument('-s', '--stb', help="Scaffold to bin file for genome-wide coverage and breadth",
-                        required=True)
-    Rflags.add_argument("-o", "--output", action="store", \
-                        help='Output prefix')
+    Rflags.add_argument("bam", help="Sorted .bam file")
+    Rflags.add_argument("fasta", help="Fasta file the bam is mapped to")
+
 
     quick_parser = subparsers.add_parser("quick_profile",formatter_class=SmartFormatter,\
                     parents = [quick_parent, parent_parser], add_help=False)
 
     # Other Parameters
     Oflags = quick_parser.add_argument_group('OTHER OPTIONS')
+    Oflags.add_argument('-s', '--stb', help="Scaffold to bin. This can be a file with each line listing a scaffold and a bin name, tab-seperated. This can also be a space-seperated list of .fasta files, with one genome per .fasta file. If nothing is provided, all scaffolds will be treated as belonging to the same genome",
+                        nargs='*', default=[])
+    Oflags.add_argument("-o", "--output", action="store", \
+                        help='Output prefix', default='QuickProfile')
     Oflags.add_argument("--breadth_cutoff", type=float, default=0.5,
-                        help='Minimum breadth to pull scaffolds')
-    Oflags.add_argument("--stringent_breadth_cutoff", type=float, default=0.01,
-                        help='Minimum breadth to let scaffold into coverm raw results')
+                        help='Minimum genome breadth to pull scaffolds')
+    Oflags.add_argument("--stringent_breadth_cutoff", type=float, default=0.00,
+                        help='Minimum breadth to let scaffold into coverm raw results (done with greater than; NOT greater than or equal to)')
 
 
     '''
@@ -303,6 +341,7 @@ def parse_args(args):
     # Other Parameters
     Oflags = other_parser.add_argument_group('OTHER OPTIONS')
     Oflags.add_argument('--old_IS', help="Convert an old inStrain version object to the newer version.")
+    Oflags.add_argument('--run_statistics', help='Generate runtime reports for an inStrain run.')
 
     '''
     ####### PARSE THE ARGUMENTS ######
