@@ -5,6 +5,7 @@ import sys
 import h5py
 import logging
 import traceback
+import warnings
 import numpy as np
 import scipy.cluster.hierarchy
 import scipy.spatial.distance as ssd
@@ -12,7 +13,8 @@ from collections import defaultdict
 
 import inStrain.SNVprofile
 import inStrain.readComparer
-import inStrain.profileUtilities
+
+import inStrain.profile.profile_utilities
 
 import matplotlib
 matplotlib.use('Agg')
@@ -30,30 +32,30 @@ import drep.d_analyze
 matplotlib.rcParams['pdf.fonttype'] = 42
 
 def mm_plot(db, left_val='breadth', right_val='coverage', title='',\
-           maxmm=15):
+           minANI=0.9):
     '''
     The input db for this is "mm_genome_info" from "makeGenomeWide" in genomeUtilities.py
     '''
-    db = db.sort_values('mm')
+    db = db.sort_values('ANI_level')
     sns.set_style('white')
 
     # breadth
     fig, ax1 = plt.subplots()
-    ax1.plot(db['mm'], db[left_val], ls='-', color='blue')
+    ax1.plot(db['ANI_level'], db[left_val], ls='-', color='blue')
     if left_val == 'breadth':
-        ax1.plot(db['mm'], estimate_breadth(db['coverage']), ls='--', color='lightblue')
+        ax1.plot(db['ANI_level'], estimate_breadth(db['coverage']), ls='--', color='lightblue')
     ax1.set_ylabel(left_val, color='blue')
-    ax1.set_xlabel('read mismatches')
+    ax1.set_xlabel('Minimum read ANI level')
     ax1.set_ylim(0,1)
 
     # coverage
     ax2 = ax1.twinx()
-    ax2.plot(db['mm'], db[right_val], ls='-', color='red')
+    ax2.plot(db['ANI_level'], db[right_val], ls='-', color='red')
     ax2.set_ylabel(right_val, color='red')
     ax2.set_ylim(0,)
 
     # asthetics
-    plt.xlim(0, maxmm)
+    plt.xlim(1, max(minANI, db['ANI_level'].min()))
 
     plt.title(title)
 
@@ -100,21 +102,23 @@ def estimate_breadth(coverage):
 
 #     return Wdb, breaks
 
-def load_windowed_metrics(IS, scaffolds, metrics=None, window_len=None, ANI_levels=[0, 100],
-                        min_scaff_len=0, report_midpoints=False):
+def load_windowed_metrics(scaffolds, s2l, rLen, metrics=None, window_len=None, ANI_levels=[0, 100],
+                        min_scaff_len=0, report_midpoints=False, covTs=False, clonTs=False,
+                        raw_linkage_table=False, cumulative_snv_table=False):
 
     if metrics is None:
-        metrics = ['coverage', 'microdiversity', 'linkage', 'snp_density']
+        metrics = ['coverage', 'nucl_diversity', 'linkage', 'snp_density']
 
     if type(metrics) != type([]):
         print("Metrics must be a list")
         return
 
     # Figure out the MMs needed
-    mms = [_get_mm(IS, ANI) for ANI in ANI_levels]
+    #rLen = IS.get_read_length()
+    mms = [_get_mm(None, ANI, rLen=rLen) for ANI in ANI_levels]
 
     # Sort the scaffolds
-    s2l = IS.get('scaffold2length')
+    #s2l = IS.get('scaffold2length')
     scaffolds = sorted(scaffolds, key=s2l.get, reverse=True)
 
     if min_scaff_len > 0:
@@ -138,25 +142,38 @@ def load_windowed_metrics(IS, scaffolds, metrics=None, window_len=None, ANI_leve
 
     dbs = []
     if 'coverage' in metrics:
-        cdb = load_windowed_coverage_or_clonality(IS, 'coverage', scaffolds, window_len, mms, ANI_levels, s2l)
+        if covTs == False:
+            logging.error("need covTs for coverage")
+            raise Exception
+        cdb = load_windowed_coverage_or_clonality('coverage', covTs, scaffolds, window_len, mms, ANI_levels, s2l)
         cdb['metric'] = 'coverage'
         dbs.append(cdb)
 #     if 'clonality' in metrics:
 #         cdb = load_windowed_coverage_or_clonality(IS, 'clonality', scaffolds, window_len, mms, ANI_levels, s2l)
 #         cdb['metric'] = 'clonality'
 #         dbs.append(cdb)
-    if 'microdiversity' in metrics:
-        cdb = load_windowed_coverage_or_clonality(IS, 'microdiversity', scaffolds, window_len, mms, ANI_levels, s2l)
-        cdb['metric'] = 'microdiversity'
+    if 'nucl_diversity' in metrics:
+        if clonTs == False:
+            logging.error("need clonTs for microdiversity")
+            raise Exception
+        cdb = load_windowed_coverage_or_clonality('nucl_diversity', clonTs, scaffolds, window_len, mms, ANI_levels, s2l)
+        cdb['metric'] = 'nucl_diversity'
         dbs.append(cdb)
     if 'linkage' in metrics:
-        cdb = load_windowed_linkage(IS, scaffolds, window_len, mms, ANI_levels, s2l)
+        if raw_linkage_table is False:
+            logging.error("need raw_linkage_table for linkage")
+            raise Exception
+        cdb = load_windowed_linkage(raw_linkage_table, scaffolds, window_len, mms, ANI_levels, s2l)
         cdb['metric'] = 'linkage'
         dbs.append(cdb)
     if 'snp_density' in metrics:
-        cdb = load_windowed_SNP_density(IS, scaffolds, window_len, mms, ANI_levels, s2l)
-        cdb['metric'] = 'snp_density'
-        dbs.append(cdb)
+        if cumulative_snv_table is False:
+            logging.error("need cumulative_snv_table for snp_density")
+            raise Exception
+        if len(cumulative_snv_table) > 0:
+            cdb = load_windowed_SNP_density(cumulative_snv_table, scaffolds, window_len, mms, ANI_levels, s2l)
+            cdb['metric'] = 'snp_density'
+            dbs.append(cdb)
 
     if len(dbs) > 0:
         Wdb = pd.concat(dbs, sort=True)
@@ -187,20 +204,22 @@ def load_windowed_metrics(IS, scaffolds, metrics=None, window_len=None, ANI_leve
     else:
         return Wdb, breaks
 
-def load_windowed_coverage_or_clonality(IS, thing, scaffolds, window_len, mms, ANI_levels, s2l):
+def load_windowed_coverage_or_clonality(thing, covTs, scaffolds, window_len, mms, ANI_levels, s2l):
     '''
     Get the windowed coverage
+
+    Pass in a clonTs for microdiversity and covTs for coverage
     '''
     if thing == 'coverage':
         item = 'covT'
-    elif thing == 'microdiversity':
+    elif thing == 'nucl_diversity':
         item = 'clonT'
     else:
         print("idk what {0} is".format(thing))
         return
 
     # Get the covTs
-    covTs = IS.get(item, scaffolds=scaffolds)
+    #covTs = IS.get(item, scaffolds=scaffolds)
 
     # Make the windows
     dbs = []
@@ -216,7 +235,7 @@ def load_windowed_coverage_or_clonality(IS, thing, scaffolds, window_len, mms, A
 
         for mm, ani in zip(mms, ANI_levels):
             if item == 'covT':
-                cov = inStrain.profileUtilities._mm_counts_to_counts_shrunk(covT, mm)
+                cov = inStrain.profile.profile_utilities.mm_counts_to_counts_shrunk(covT, mm)
                 if len(cov) == 0:
                     continue
                 db = _gen_windowed_cov(cov, window_len, sLen=s2l[scaffold])
@@ -245,10 +264,10 @@ def load_windowed_coverage_or_clonality(IS, thing, scaffolds, window_len, mms, A
         Wdb = pd.DataFrame()
     return Wdb#, breaks
 
-def load_windowed_linkage(IS, scaffolds, window_len, mms, ANI_levels, s2l, on='r2'):
+def load_windowed_linkage(Ldb, scaffolds, window_len, mms, ANI_levels, s2l, on='r2'):
 
     # Get the linkage table
-    Ldb = IS.get('raw_linkage_table')
+    #Ldb = IS.get('raw_linkage_table')
     Ldb = Ldb[Ldb['scaffold'].isin(scaffolds)].sort_values('mm')
     got_scaffolds = set(Ldb['scaffold'].unique())
 
@@ -265,7 +284,7 @@ def load_windowed_linkage(IS, scaffolds, window_len, mms, ANI_levels, s2l, on='r
             ldb = Ldb[Ldb['scaffold'] == scaffold]
 
         for mm, ani in zip(mms, ANI_levels):
-            db = ldb[ldb['mm'] <= mm].drop_duplicates(subset=['scaffold', 'position_A', 'position_B'], keep='last')
+            db = ldb[ldb['mm'] <= int(mm)].drop_duplicates(subset=['scaffold', 'position_A', 'position_B'], keep='last')
             cov = db.set_index('position_A')[on].sort_index()
 
             db = _gen_windowed_cov(cov, window_len, sLen=s2l[scaffold], full_len=False)
@@ -286,9 +305,9 @@ def load_windowed_linkage(IS, scaffolds, window_len, mms, ANI_levels, s2l, on='r
         Wdb = pd.DataFrame()
     return Wdb
 
-def load_windowed_SNP_density(IS, scaffolds, window_len, mms, ANI_levels, s2l):
+def load_windowed_SNP_density(Ldb, scaffolds, window_len, mms, ANI_levels, s2l):
     # Get the table
-    Ldb = IS.get('cumulative_snv_table')
+    #Ldb = IS.get('cumulative_snv_table')
     Ldb = Ldb[Ldb['scaffold'].isin(scaffolds)].sort_values('mm')
 
     got_scaffolds = list(Ldb['scaffold'].unique())
@@ -306,8 +325,8 @@ def load_windowed_SNP_density(IS, scaffolds, window_len, mms, ANI_levels, s2l):
             ldb = Ldb[Ldb['scaffold'] == scaffold]
 
         for mm, ani in zip(mms, ANI_levels):
-            db = ldb[ldb['mm'] <= mm].drop_duplicates(subset=['scaffold', 'position'], keep='last')
-            cov = db.set_index('position')['refBase'].sort_index()
+            db = ldb[ldb['mm'] <= int(mm)].drop_duplicates(subset=['scaffold', 'position'], keep='last')
+            cov = db.set_index('position')['ref_base'].sort_index()
 
             db = _gen_windowed_cov(cov, window_len, sLen=s2l[scaffold], full_len='count')
             db['scaffold'] = scaffold
@@ -370,14 +389,16 @@ def load_windowed_SNP_density(IS, scaffolds, window_len, mms, ANI_levels, s2l):
 #     #plt.ylabel('coverage')
 #     plt.xlim(0, Wdb['adjusted_start'].max())
 
-def _get_mm(IS, ANI):
+def _get_mm(IS, ANI, rLen = None):
     '''
     Get the mm corresponding to an ANI level in an IS
     '''
     if ANI > 1:
         ANI = ANI / 100
 
-    rLen = IS.get('read_report')['mean_pair_length'].tolist()[0]
+    if rLen == None:
+        rLen = IS.get_read_length()
+        #rLen = IS.get('mapping_info')['mean_pair_length'].tolist()[0]
     mm = int(round((rLen - (rLen * ANI))))
     return mm
 
@@ -441,7 +462,9 @@ def iterate_chunks_series(d, chunkSize=100, sLen=False):
     numberChunks = sLen // chunkSize + 1
     for i in range(numberChunks):
         #print(i, int(i*chunkSize), int((i+1)*chunkSize), len(d))
-        yield (d.loc[int(i*chunkSize):int((i+1)*chunkSize)])
+        start = int(i*chunkSize)
+        end = int((i+1)*chunkSize)
+        yield (d.loc[start:end])
 
 def plot_RC_SNPs(hd5_loc, scaffolds, s2l, mm_level=None, samples=None):
     '''
@@ -574,7 +597,7 @@ def genomeWide_microdiveristy_metrics_plot(Wdb, breaks, title=''):
         ax[0] = plt.gca()
 
     i = 0
-    for metric in ['linkage', 'snp_density', 'coverage', 'microdiversity']:
+    for metric in ['linkage', 'snp_density', 'coverage', 'nucl_diversity']:
     #for metric, wdb in Wdb.groupby('metric'):
         if metric not in set(Wdb['metric'].tolist()):
             continue
@@ -649,23 +672,24 @@ def prepare_read_ani_dist_plot(IS):
         for genome, df in Odb.groupby('genome'):
             table['mm'].append(mm)
             table['genome'].append(genome)
-            table['true_length'].append(int(b2l[genome]))
+            table['length'].append(int(b2l[genome]))
 
             for col in ['coverage']:
                 table[col].append(sum(x * y for x, y in zip(df[col], df['length'])) / b2l[genome])
     db = pd.DataFrame(table)
 
     # Add the number of read-pairs
-    readLen = int(IS.get('read_report')['mean_pair_length'].tolist()[0])
+    #readLen = int(IS.get('mapping_info')['mean_pair_length'].tolist()[0])
+    readLen = int(IS.get_read_length())
     db['read_length'] = readLen
     db['mm'] = db['mm'].astype(int)
-    db['read_pairs'] = [int((x*y) / (readLen * 2)) for x, y in zip(db['coverage'], db['true_length'])]
+    db['read_pairs'] = [int((x*y) / (readLen * 2)) for x, y in zip(db['coverage'], db['length'])]
     db['ANI_level'] = [(readLen - mm)/readLen for mm in db['mm']]
 
     return db
 
 def major_allele_freq_plot(db, title=''):
-    db['major_allele_freq'] = [max(x, y) for x, y in zip(db['varFreq'], db['refFreq'])]
+    db['major_allele_freq'] = [max(x, y) for x, y in zip(db['var_freq'], db['ref_freq'])]
     sns.distplot(db['major_allele_freq'], bins=np.arange(0.5, 1, 0.01), kde=False)
 
     plt.xlim(0.5, 1)
@@ -680,6 +704,7 @@ def linkage_decay_plot(db, chunkSize=5, min_vals=5, title=''):
     max_d = db['distance'].max()
     table = defaultdict(list)
     numberChunks = max_d // chunkSize + 1
+    db['distance'] = db['distance'].astype(int)
     for i in range(numberChunks):
         d = db[(db['distance'] >= int(i*chunkSize)) & (db['distance'] < int((i+1)*chunkSize))]
 
@@ -735,13 +760,14 @@ def read_filtering_plot(db, title=''):
     # Annotate every single Bar with its value, based on it's width
     offset = db['value'].max() / 12
     total = db[db['variable'] == 'Total mapped pairs']['value'].tolist()[0]
-    for i, p in enumerate(ax.patches):
-        if i == 0:
-            continue
-        width = p.get_width()
-        plt.text(offset + p.get_width(), p.get_y()+0.55*p.get_height(),
-                 '{:1.0f}%'.format((width/total)*100),
-                 ha='center', va='center')
+    if total > 0:
+        for i, p in enumerate(ax.patches):
+            if i == 0:
+                continue
+            width = p.get_width()
+            plt.text(offset + p.get_width(), p.get_y()+0.55*p.get_height(),
+                     '{:1.0f}%'.format((width/total)*100),
+                     ha='center', va='center')
 
     plt.title(title)
 
@@ -760,7 +786,7 @@ def scaffold_inspection_plot(Wdb, breaks, midpoints, title=''):
         ax[0] = plt.gca()
 
     i = 0
-    for metric in ['linkage', 'snp_density', 'coverage', 'microdiversity']:
+    for metric in ['linkage', 'snp_density', 'coverage', 'nucl_diversity']:
     #for metric, wdb in Wdb.groupby('metric'):
         if metric not in set(Wdb['metric'].tolist()):
             continue
@@ -822,6 +848,7 @@ def linkage_decay_type(Odb, chunkSize=5, min_vals=2, title=''):
         else:
             db = Odb[Odb['link_type'] == lt]
 
+        db['distance'] = db['distance'].astype(int)
         for i in range(numberChunks):
             d = db[(db['distance'] >= int(i*chunkSize)) & (db['distance'] < int((i+1)*chunkSize))]
 
@@ -863,7 +890,7 @@ def calc_link_type(row, k2t):
         return np.nan
 
 def gene_histogram_plot(db, title=''):
-    COLS = ['coverage', 'microdiversity', 'SNPs_per_bp']
+    COLS = ['coverage', 'nucl_diversity', 'SNPs_per_bp']
 
      # Get set up for multiple rows
     i = len(set(db.columns).intersection(set(COLS)))
@@ -911,7 +938,10 @@ def plot_readComparerer_dendrograms(gdb, title, cluster_method='single', thresh=
         linkage = scipy.cluster.hierarchy.linkage(arr, method=cluster_method)
         fclust = scipy.cluster.hierarchy.fcluster(linkage, thresh,
                             criterion='distance')
-        Cdb = drep.d_cluster._gen_cdb_from_fclust(fclust, names)
+        try:
+            Cdb = drep.d_cluster._gen_cdb_from_fclust(fclust, names)
+        except AttributeError:
+            Cdb = drep.d_cluster.utils._gen_cdb_from_fclust(fclust, names)
 
         if thing == 'av_ani':
             name2cluster = Cdb.set_index('genome')['cluster'].to_dict()
@@ -919,8 +949,10 @@ def plot_readComparerer_dendrograms(gdb, title, cluster_method='single', thresh=
 
         # Plot
         plt.sca(t2a[thing])
-        _make_RC_dendrogram(linkage, names, xlabel=t2c[thing], subtitle_string=title, threshold=t2t[thing],
-                           name2color=name2color)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            _make_RC_dendrogram(linkage, names, xlabel=t2c[thing], subtitle_string=title, threshold=t2t[thing],
+                               name2color=name2color)
 
     # Fix axis labels 1
     for thing in ['av_cov', 'av_ani']:
@@ -1066,7 +1098,7 @@ def plot_genome(genome, IS, **kwargs):
     GWdb = kwargs.get('GWdb', False)
 
     # FILTER BY BREADTH
-    mb = kwargs.get('minimum_breadth', 0)
+    mb = float(kwargs.get('minimum_breadth', 0))
     if mb > 0:
         if GWdb is False:
             GWdb = inStrain.genomeUtilities.genomeWideFromIS(IS, 'scaffold_info', mm_level=False)
@@ -1076,7 +1108,7 @@ def plot_genome(genome, IS, **kwargs):
             breadth = GWdb[GWdb['genome'] == genome]['breadth'].tolist()[0]
         except:
             breadth = 0
-        if breadth < mb:
+        if float(breadth) < mb:
             return False
 
     # FILTER BY GENOME LIST
@@ -1127,12 +1159,19 @@ def main(args):
     # Cache needed data
     if IS_TYPE == 'IS':
         try:
-            kwargs['GWdb'] = inStrain.genomeUtilities.genomeWideFromIS(IS, 'scaffold_info', mm_level=True)
+            kwargs['GWdb'] = IS.get('genome_level_info')
         except:
             logging.error("Cannot cache scaffold info - you don't have all required information. You need to run inStrain genome_wide first")
             if debug:
                 traceback.print_exc()
 
+    # Keep cacheing
+    if (('2' in to_plot) | ('7' in to_plot)):
+        kwargs['covT'] = IS.get('covT')
+        kwargs['clonT'] = IS.get('clonT')
+        kwargs['raw_linkage_table'] = IS.get('raw_linkage_table')
+        kwargs['cumulative_snv_table'] = IS.get('cumulative_snv_table')
+
     # 1) MM plot
     if '1' in to_plot:
         try:
@@ -1224,6 +1263,8 @@ def main(args):
                 traceback.print_exc()
                 logging.debug(traceback.format_exc())
 
+    logging.debug("Plotting plots finished")
+
 def validate_input(args):
     '''
     Validate and mess with the arguments a bit
@@ -1278,9 +1319,14 @@ def mm_plot_from_IS(IS, plot_dir=False, **kwargs):
     # Load the required data
     try:
         Mdb = kwargs.get('GWdb', False)
-        if Mdb is False:
-            Mdb = inStrain.genomeUtilities.genomeWideFromIS(IS, 'scaffold_info', mm_level=True)
         assert len(Mdb) > 0
+
+        # Add the number of read-pairs
+        #readLen = int(IS.get('mapping_info')['mean_pair_length'].tolist()[0])
+        readLen = int(IS.get_read_length())
+        Mdb['read_length'] = readLen
+        Mdb['mm'] = Mdb['mm'].astype(int)
+        Mdb['ANI_level'] = [(readLen - mm)/readLen for mm in Mdb['mm']]
     except:
         logging.error("Skipping plot 1 - you don't have all required information. You need to run inStrain genome_wide first")
         traceback.print_exc()
@@ -1315,6 +1361,16 @@ def genome_plot_from_IS(IS, plot_dir=False, **kwargs):
         for s, b in stb.items():
             b2s[b].append(s)
         assert len(b2s.keys()) > 0
+
+        # Load the cache
+        covTs = kwargs.get('covT')#, IS.get('covT'))
+        clonTs = kwargs.get('clonT')#, IS.get('clonT'))
+        raw_linkage_table = kwargs.get('raw_linkage_table')#, IS.get('raw_linkage_table'))
+        cumulative_snv_table = kwargs.get('cumulative_snv_table')#, IS.get('cumulative_snv_table'))
+        scaffold2length = IS.get('scaffold2length')
+        rl = IS.get_read_length()
+        profiled_scaffolds = set(scaffold2length.keys())
+
     except:
         logging.error("Skipping plot 2 - you don't have all required information. You need to run inStrain genome_wide first")
         traceback.print_exc()
@@ -1325,10 +1381,18 @@ def genome_plot_from_IS(IS, plot_dir=False, **kwargs):
     name = 'genomeWide_microdiveristy_metrics.pdf'
     pp = PdfPages(plot_dir + name)
 
+
     for genome, scaffolds in b2s.items():
         if not plot_genome(genome, IS, **kwargs):
             continue
-        Wdb, breaks = load_windowed_metrics(IS, scaffolds, ANI_levels=[0, 100])
+        present_scaffolds = list(set(scaffolds).intersection(set(profiled_scaffolds)))
+        Wdb, breaks, midpoints = load_windowed_metrics(present_scaffolds,
+                                scaffold2length,
+                                rl,
+                                report_midpoints=True,
+                                covTs=covTs, clonTs=clonTs,
+                                raw_linkage_table=raw_linkage_table,
+                                cumulative_snv_table=cumulative_snv_table)
         genomeWide_microdiveristy_metrics_plot(Wdb, breaks, title=genome)
         fig = plt.gcf()
         fig.set_size_inches(8, 5)
@@ -1377,6 +1441,8 @@ def allele_freq_plot_from_IS(IS, plot_dir=False, **kwargs):
     # Load the required data
     try:
         db = IS.get('cumulative_snv_table')
+        if len(db) == 0:
+            return
         db = db.sort_values('mm').drop_duplicates(subset=['scaffold', 'position'], keep='last')\
                     .sort_index().drop(columns=['mm'])
         db = db[(db['cryptic'] == False)]
@@ -1450,11 +1516,46 @@ def linkage_decay_from_IS(IS, plot_dir=False, **kwargs):
     #plt.show()
     plt.close('all')
 
+# def read_filtering_from_IS(IS, plot_dir=False, **kwargs):
+#     # Load the required data
+#     try:
+#         # Prepare
+#         Mdb = inStrain.genomeUtilities.genomeWideFromIS(IS, 'mapping_info', mm_level=True)
+#         print(Mdb)
+#         assert len(Mdb) > 0
+#     except:
+#         logging.error("Skipping plot 6 - you don't have all required information. You need to run inStrain genome_wide first")
+#         traceback.print_exc()
+#         return
+#
+#     # Make the plot
+#     logging.info("Plotting plot 6")
+#     name = 'ReadFiltering_plot.pdf'
+#     pp = PdfPages(plot_dir + name)
+#
+#     for genome, mdb in Mdb.groupby('genome'):
+#         if not plot_genome(genome, IS, **kwargs):
+#             continue
+#         read_filtering_plot(mdb, title=genome)
+#         fig = plt.gcf()
+#         fig.set_size_inches(6, 4)
+#         fig.tight_layout()
+#         pp.savefig(fig)#, bbox_inches='tight')
+#         #plt.show()
+#         plt.close(fig)
+#
+#     # Save the figure
+#     pp.close()
+#     #plt.show()
+#     plt.close('all')
+
 def read_filtering_from_IS(IS, plot_dir=False, **kwargs):
     # Load the required data
     try:
         # Prepare
-        Mdb = inStrain.genomeUtilities.genomeWideFromIS(IS, 'read_report', mm_level=True)
+        Mdb = IS.get('mapping_info')
+        Mdb = Mdb[Mdb['scaffold'] == 'all_scaffolds']
+        Mdb['genome'] = 'all_scaffolds'
         assert len(Mdb) > 0
     except:
         logging.error("Skipping plot 6 - you don't have all required information. You need to run inStrain genome_wide first")
@@ -1466,20 +1567,16 @@ def read_filtering_from_IS(IS, plot_dir=False, **kwargs):
     name = 'ReadFiltering_plot.pdf'
     pp = PdfPages(plot_dir + name)
 
-    for genome, mdb in Mdb.groupby('genome'):
-        if not plot_genome(genome, IS, **kwargs):
-            continue
-        read_filtering_plot(mdb, title=genome)
-        fig = plt.gcf()
-        fig.set_size_inches(6, 4)
-        fig.tight_layout()
-        pp.savefig(fig)#, bbox_inches='tight')
-        #plt.show()
-        plt.close(fig)
+    read_filtering_plot(Mdb, title='all scaffolds')
+
+    fig = plt.gcf()
+    fig.set_size_inches(6, 4)
+    fig.tight_layout()
+    pp.savefig(fig)
+    plt.close(fig)
 
     # Save the figure
     pp.close()
-    #plt.show()
     plt.close('all')
 
 def scaffold_inspection_from_IS(IS, plot_dir=False, **kwargs):
@@ -1490,6 +1587,16 @@ def scaffold_inspection_from_IS(IS, plot_dir=False, **kwargs):
         for s, b in stb.items():
             b2s[b].append(s)
         assert len(b2s.keys()) > 0
+
+        # Load the cache
+        covTs = kwargs.get('covTs', IS.get('covT'))
+        clonTs = kwargs.get('clonTs', IS.get('clonT'))
+        raw_linkage_table = kwargs.get('raw_linkage_table', IS.get('raw_linkage_table'))
+        cumulative_snv_table = kwargs.get('cumulative_snv_table', IS.get('cumulative_snv_table'))
+        scaffold2length = IS.get('scaffold2length')
+        rl = IS.get_read_length()
+        profiled_scaffolds = set(scaffold2length.keys())
+
     except:
         logging.error("Skipping plot 7 - you don't have all required information. You need to run inStrain genome_wide first")
         traceback.print_exc()
@@ -1503,7 +1610,14 @@ def scaffold_inspection_from_IS(IS, plot_dir=False, **kwargs):
     for genome, scaffolds in b2s.items():
         if not plot_genome(genome, IS, **kwargs):
             continue
-        Wdb, breaks, midpoints = load_windowed_metrics(IS, scaffolds, report_midpoints=True)
+        present_scaffolds = list(set(scaffolds).intersection(set(profiled_scaffolds)))
+        Wdb, breaks, midpoints = load_windowed_metrics(present_scaffolds,
+                                scaffold2length,
+                                rl,
+                                report_midpoints=True,
+                                covTs=covTs, clonTs=clonTs,
+                                raw_linkage_table=raw_linkage_table,
+                                cumulative_snv_table=cumulative_snv_table)
         scaffold_inspection_plot(Wdb, breaks, midpoints, title=genome)
         fig = plt.gcf()
         fig.tight_layout()
@@ -1528,6 +1642,8 @@ def linkage_decay_type_from_IS(IS, plot_dir=False, **kwargs):
 
         SNdb = IS.get('SNP_mutation_types')
         assert SNdb is not None
+        if len(SNdb) == 0:
+            return
         SNdb['key'] = ["{0}:{1}".format(s, p) for s, p in zip(SNdb['scaffold'], SNdb['position'])]
         k2t = SNdb.set_index('key')['mutation_type'].to_dict()
 
@@ -1568,7 +1684,7 @@ def gene_histogram_from_IS(IS, plot_dir=False, **kwargs):
         stb = IS.get('scaffold2bin')
         Gdb = inStrain.genomeUtilities._add_stb(db, stb)
         if 'clonality' in Gdb.columns:
-            Gdb['microdiversity'] = 1 - Gdb['clonality']
+            Gdb['nucl_diversity'] = 1 - Gdb['clonality']
         assert len(Gdb) > 0
     except:
         logging.error("Skipping plot 9 - you don't have all required information. You need to run inStrain profile_genes first")
